------------------------------------------------------------------------------
--                                                                          --
--                            Matreshka Project                             --
--                                                                          --
--                               Web Framework                              --
--                                                                          --
--                        Runtime Library Component                         --
--                                                                          --
------------------------------------------------------------------------------
--                                                                          --
-- Copyright © 2014-2017, Vadim Godunko <vgodunko@gmail.com>                --
-- All rights reserved.                                                     --
--                                                                          --
-- Redistribution and use in source and binary forms, with or without       --
-- modification, are permitted provided that the following conditions       --
-- are met:                                                                 --
--                                                                          --
--  * Redistributions of source code must retain the above copyright        --
--    notice, this list of conditions and the following disclaimer.         --
--                                                                          --
--  * Redistributions in binary form must reproduce the above copyright     --
--    notice, this list of conditions and the following disclaimer in the   --
--    documentation and/or other materials provided with the distribution.  --
--                                                                          --
--  * Neither the name of the Vadim Godunko, IE nor the names of its        --
--    contributors may be used to endorse or promote products derived from  --
--    this software without specific prior written permission.              --
--                                                                          --
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      --
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        --
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR    --
-- A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     --
-- HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   --
-- SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED --
-- TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   --
-- PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF   --
-- LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING     --
-- NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS       --
-- SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.             --
--                                                                          --
------------------------------------------------------------------------------
--  $Revision: 5735 $ $Date: 2017-01-28 16:37:29 +0300 (Sat, 28 Jan 2017) $
------------------------------------------------------------------------------
with League.Strings;

with Servlet.Contexts;
with Servlet.Context_Listeners;

package Servlet.Application is

   procedure Initialize
    (Application_Name    : League.Strings.Universal_String;
     Application_Version : League.Strings.Universal_String;
     Organization_Name   : League.Strings.Universal_String;
     Organization_Domain : League.Strings.Universal_String);
   --  Initialize servlet container. It detects run of application under
   --  FastCGI environment and use it when possible, or fallback to start as
   --  standalone AWS server when it supports was enabled at build time.

   procedure Execute;

   procedure Finalize;

   procedure Add_Listener
    (Listener : not null
       Servlet.Context_Listeners.Servlet_Context_Listener_Access);
   --  Adds Servlet_Context_Listener. Context_Initialized subprogram of this
   --  interface will be called during execution of Initialize subprogram; and
   --  Context_Destoyed subprogram will be called during execution of Finalize
   --  subprogram.

   function Get_Servlet_Context return
     not null Servlet.Contexts.Servlet_Context_Access;
   --  Returns servlet context of the initialized servlet container.
   --  Application should use this context to add and configure servlets and
   --  filters.

end Servlet.Application;
